/* $Id: filter.h 408 2010-05-05 12:09:42Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef FILTER_H
#define FILTER_H

// std stuff.
#include <vector>
#include <string>

// wx stuff.
#include <wx/string.h>
#include <wx/regex.h>


namespace Cfg
{
	/// One filter element containing parts ANDed together.
	class Filter
	{
	public:
		
		/// Constructor.
		Filter();
		/// Copy constructor.
		Filter(Filter const& old);
		/// Destructor.
		~Filter();
		
		/// Area give by a a/n/n/n/n-rule.
		struct Area
		{
			/// Northern limit for latitude.
			double latN;
			/// Western limit for longitude.
			double lonW;
			/// Southern limit for latitude.
			double latS;
			/// Eastern lmit for longitude.
			double lonE;
		};
		
		/// Range given by a r/n/n/n-rule.
		struct Range
		{
			double lat;
			double lon;
			double dist;
		};

		/// Add given area element to this filter.
		void addArea(Area const& a);
		
		/// Add given range element to this filter.
		void addRange(Range const& r);
		
		/// Add given buddy regexp pattern to this filter.
		void addBuddyPattern(wxString const& p);
		
		/// Tells wether this filter has patterns in buddylist or not.
		bool hasBuddyPatterns() const;
		
		/// Get amount of parts in this filter.
		unsigned int getPartCount() const;
		
		
		/// Return true, if filter accepts (passes) given values.
		bool accepts(std::string const& call, double const& lon, double const& lat) const;
		
		/// Returns true, if filter has only callsign-based parts and given callsign passes it.
		bool accepts(std::string const& call) const;
		
	private:

		// Calculate distance in km from location to another.
		double distance(double lon0, double lat0, double lon1, double lat1) const;
		
		/// Areas a packet is going to be matched against.
		std::vector<Area> areas_;

		// Ranges.
		std::vector<Range> ranges_;
		
		/// Regeps for matching source callsign of a packet.
		std::vector<wxRegEx const*> buddyRegs_;
		/// Regexps as strings for regexp recreation.
		std::vector<wxString> buddyPatterns_;
		
	};
	
} // namespace Cfg

	
#endif // FILTER_H
